#include "extension_db.h"
#include <string.h>

extension_db::extension_db()
{
    init();
}

int extension_db::extension_index(const char *str)
{
    int idex = 0;
    if(str[0]){
	idex += idx(str[0]);
	if(str[1]){
	    idex += 37 * idx(str[1]);
	    if(str[2]){
		idex += 37 * 37 * idx(str[2]);
	    }
	}
    }
    return idex;
}


void extension_db::add(const char *str,struct ent e)
{
    /* Find the extension first */
    const char *cc = rindex(str,'.');
    if(!cc) return;
    cc++;
    if(strlen(cc)>3){			// looks like extension is too long?
	if(cc[3]!=':'){			// is this a virtual file name? 
	    return;			// no; extension is invalid
	}
    }   
    for(int i=0;i<4;i++){
	db[extension_index(cc)].partial[i] += e.partial[i];
	db[extension_index(cc)].complete[i] += e.complete[i];
    }
}

void extension_db::init()
{
    memset(db,0,sizeof(db));
}

int  extension_db::count(struct ent e)
{
    int count=0;
    for(int i=0;i<4;i++){
	count += e.partial[i] + e.complete[i];
    }
    return count;
}

void extension_db::report(FILE *outfile,struct ent e)
{
    for(int i=0;i<4;i++){
	fprintf(outfile,"%5d/%-5d ",e.partial[i],e.complete[i]);
    }
}

void extension_db::report(FILE *outfile)
{
    int i,j,k;
    char *str = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";

    fprintf(outfile,"  extension     ");
    for(i=0;i<4;i++){
	fprintf(outfile,"  level %d   ",i);
    }
    fputc('\n',outfile);
    for(i=0;i<36;i++){
	for(j=0;j<36;j++){
	    for(k=0;k<36;k++){
		char ext[4];

		ext[0] = str[i];
		ext[1] = str[j];
		ext[2] = str[k];
		ext[3] = 0;
		
		struct ent e = db[extension_index(ext)];
		if(count(e)>0){
		    fprintf(outfile,"%c%c%c: part/comp: ",str[i],str[j],str[k]);
		    report(outfile,e);
		    fputc('\n',outfile);
		}
	    }
	}
    }
}
