/*
 * fatdump.h:
 * 
 * Copyright (c) 2002, Massachusetts Institute of Technology
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * Defines the classes used by the fatdump system.
 * 
 * Classes in this file:
 *
 * fatdumper - the actual forensic file system.
 * fat_BPB - Information abstracted from a BPB
 * fat_dirent - fat directory entry
 */

#include <sys/types.h>

#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>

#include <ctype.h>
#include <memory.h>
#include <string.h>

#include <time.h>

#ifdef WIN32
#include <io.h>
#include <conio.h>
#endif

#ifdef UNIX
#include <sys/uio.h>
#include <unistd.h>
#endif

extern int debug;

void gui_main(class ffs *ffs);

extern int opt_q;			// quiet mode
extern int opt_v;			// verbose
extern int opt_i;			// interactive
extern int opt_k;			// limit to first 100k blocks

/* Define annotations that this system uses */
#define A_FREE 0x00
#define A_USED01  0x01			// used by level0 or level1 data
#define A_USED2   0x02			// used by level2 data
#define A_USED_DIR 0x04		// has been used for displaying a directory; prevents loops

/* Possible status for a file */
#define F_COMPLETE 1			// entire file is readable
#define F_PARTIAL  2			// file is partially there
#define F_NONE     3			// none of file remains

extern extension_db extensions;	// complete extensions
extern sorted times;

extern class blockmap sectormap;			// our blockmap
extern class blockmap clustermap;			// our blockmap

extern int opt_extensions;		// compute extensions database?
extern int opt_dir;			// print long information, dos DIR style
extern int opt_R;
extern int opt_lsl;			// ls -l
extern int opt_no0;			// omit level 0 data
extern int opt_no1;			// omit level 1 data
extern int opt_no2;			// no level 2 files should be displayed
extern int opt_no3;			// no level 3 files shoudl be displayed
extern int opt_showLevel;
extern int opt_showClusterChain;	// show the clusterchain for each file

void map_level01(class ffs &fs,const char *basedir,pos loc,int blocks);
int scan_dir(class ffs &fs,const char *basedir,pos loc,int blocks);
