#include "StdAfx.h"
#include "ffs.h"
#include <stdlib.h>
#include <string.h>
#include "gui.h"			// this needs to come out
#include "fatdump.h"

#ifndef MIN
#define MIN(x,y) ((x)<(y) ? (x) : (y))
#endif


/*
 * ffs.cpp:
 * 
 * The Forensics file system.
 * This module puts together all of the other modules.
 */

/****************************************************************
 ****************************************************************
 ****************************************************************/
								 
int ffs::debug = 0;
int ffs::opt_q = 0;
int ffs::opt_v = 0;

char *ffs::itocomma(char *buf,unsigned i)
{
    char *tmp;

    buf[0] = 0;
    while(i>0){
	int rem = i % 1000;
	i = i / 1000;

	tmp = strdup(buf);
	if(i>0){
	    sprintf(buf,",%03d%s",rem,tmp);
	}
	else{
	    sprintf(buf,"%d%s",rem,tmp);
	}
	free(tmp);
    }
    return buf;
}

/****************************************************************
/****************************************************************
 ****************************************************************
 ****************************************************************/


void dump_tm(FILE *out,struct tm *tm)
{
    fprintf(out,"%4d-%2d-%2d %2d:%02d:%02d",
	    tm->tm_year+1900,
	    tm->tm_mon+1,
	    tm->tm_mday,
	    tm->tm_hour,
	    tm->tm_min,
	    tm->tm_sec);
}



#ifdef __APPLE__
const char *strcasestr(const char *big, const char *little)
{
    while(*big){
	if(strcasecmp(big,little)==0) return big;
	big++;
    }
    return 0;
}
#endif

ffs::ffs(class img *img_)
{
    img = img_;
}


bool ffs::analyze_islevel1(const char *filename)
{
    if(strncasecmp(filename,"recycle",7)==0) return true; // looks like it is a recycler
    if(strcasestr(filename,"temporary")) return true; // looks like it is a temporary file
    if(strcasestr(filename,".tmp")) return true; // looks like *.TMP
    return false;
}

bool ffs::find_BPB(sector_t start)
{
    bool  first = true;
    class fat_BPB temp_bpb;		// where we read temp buf
    uchar  buf[512];
    
    while(img->readblock(start,buf)==512){
	if(debug) printf("find_BPB(%d)\n",start);
	    
	if(temp_bpb.readbuf(buf)){	// hey, I found it!

	    int testflag = 0;
	    int matched_blocks=0;
	    int zero_blocks=0;
	    enum {
		FOUND_BPB = 0x01,
		FAT_MATCH = 0x02,
		FAT_VALIDATED = 0x04,
		TEST_ALL = FOUND_BPB|FAT_MATCH|FAT_VALIDATED
	    };

	    sector_t fat0_start = start+temp_bpb.BPB_RsvdSecCnt;
	    sector_t fat1_start = fat0_start+temp_bpb.FATSz;

	    if(opt_v>0) printf("BPB found at sector %d\n",start);
	    if(opt_v>1) temp_bpb.print(stdout);

	    testflag = FOUND_BPB;

	    bool anyway = false;

	    /* See if the fats match */
	    if(opt_v>1){
		printf("Checking %d blocks of FAT\n",temp_bpb.FATSz);
	    }
	    if(fat_FAT::fat_match(this,
				  fat0_start,fat1_start,
				  temp_bpb.FATSz,
				  &matched_blocks,&zero_blocks)){
		if(opt_v>0) printf("FATs MATCH! (%d blocks; %d all zero)\n",
				   matched_blocks,zero_blocks);
		testflag |= FAT_MATCH;
	    }
	    else {
		if(opt_q==0){
		    printf("BPB found; FATs do not match "
			   "(%d blocks matching; %d all zero)\n",
			   matched_blocks,zero_blocks);

		    printf("%s %d\n",temp_bpb.BS_VolLab,strlen(temp_bpb.BS_VolLab));
		}
		if(strcmp(temp_bpb.BS_VolLab,"COMPRESSED ")==0){
		    if(opt_q==0) printf(">> this is a compressed volume! <<\n");
		    anyway = true;
		}
	    }


	    /* Try to read fat0 and see if it validates */
	    fat0.read(this,fat0_start,&temp_bpb);
	    if(fat0.validate()){
		if(opt_v>0) printf("FAT0 VALIDATES!\n");
		testflag |= FAT_VALIDATED;
	    }


	    if(testflag != TEST_ALL){
		printf("using anyway...\n");
		anyway = true;
	    }
		
	    if(testflag == TEST_ALL || anyway ){
		bpb = temp_bpb;
		BPB_loc = start;
		bpb.set_image_offset(BPB_loc-1); // set this image offset
		return true;
	    }

	    if(first){
		bpb     = temp_bpb;		// make a copy
		BPB_loc = start;		// remember where we found it
		first   = false;
	    }

	    printf("Looking for another...\n");
	}
	if(kbhit()){
	    int ch = keychar();
	    if(ch==('t'-'a'+1)){
		printf("Scanning for BPB at block %d of %d\n",
		       start,img->image_size/512);
	    }
	}
	start++;			// check the next sector
	if(opt_q==0 && start%10000==0){
	    printf("\r%6d...",start);
	    fflush(stdout);
	}
	if(opt_k && start>100000){
	    return false;		// limit to the first 100k blocks
	}
    }
    return false;			// ran out of image?
}

/* Get the contents of a single cluster into a buffer.
 */
int ffs::get_cluster(cluster_t clusterstart,uchar *buf,int buflen)
{
    sector_t sector = bpb.cluster2sector(clusterstart);
    if(buflen > bpb.BPB_SecPerClus*512){ // if buffer is larger than we need, shorten it
	buflen = bpb.BPB_SecPerClus*512;
    }
    return img->readblocks(sector,buf,buflen/512);
}


/* Write the cluster chain to either a FILE or a pointer.
 * If ptr!=0, it is allocated.
 * TK: Need to instrument this so that the same block is not written twice
 * (which would be a loop).
 * Use a sparse array to keep track of what has been written!
 */
int ffs::write_clusterchain(FILE *f,uchar **ptr,
				cluster_t clusterstart,unsigned long fsize)
{
    uchar buf[512];			// block to read
    ulong bytes_written=0;

    if(ptr) (*ptr) = (uchar *)malloc(0);

    do {

	sector_t sector = bpb.cluster2sector(clusterstart);

	for(int block_in_cluster=0;
	    block_in_cluster < bpb.BPB_SecPerClus;
	    block_in_cluster++){

	    sector_t sector_to_read = sector + block_in_cluster;
	    memset(buf,'.',sizeof(buf));

	    if(img->readblock(sector_to_read,buf)==0){
		fprintf(stderr,"  >> sector %d not in image <<\n",sector_to_read);
		return bytes_written;
	    }
	    int count = MIN(fsize,512);

	    if(f){
		fwrite(buf,1,count,f);
	    }
	    if(ptr){
		(*ptr) = (uchar *)realloc(*ptr,bytes_written + count);
		memcpy((*ptr)+bytes_written,buf,count);
	    }
	    fsize -= count;
	    bytes_written += count;

	    if(fsize==0) return bytes_written;
	}

	/* Now find the next sector */
	clusterstart = fat0.next_cluster(clusterstart);
    } while(clusterstart!=fat_FAT::fatEOF);
    return bytes_written;
}

void   ffs::fprint_cluster(FILE *f,cluster_t cluster)
{
    uchar buf[512];
    sector_t sector = bpb.cluster2sector(cluster);

    if(img->readblock(sector,buf)==0){
	fwrite(buf,1,512,f);
    }
}

class img::sectorchain &ffs::get_sectorchain(cluster_t clusterstart)
{
    class img::sectorchain *chain = new img::sectorchain;
    do {
	sector_t sector = bpb.cluster2sector(clusterstart);

	for(int block_in_cluster=0;
	    block_in_cluster < bpb.BPB_SecPerClus;
	    block_in_cluster++){
	    
	    sector_t num = sector + block_in_cluster;
	    if(chain->add(num)==false){
		fprintf(stderr,"cross-linked at sector %d\n",num);
		return *chain;		// cross-linked! go home
	    }
	}
	clusterstart = fat0.next_cluster(clusterstart);
    } while(clusterstart!=fat_FAT::fatEOF);
    return *chain;
}

class ffs_chain &ffs::get_clusterchain(cluster_t clusterstart)
{
    class ffs_chain *chain = new img::sectorchain;
    do {
	if(chain->add(clusterstart)==false){
	    fprintf(stderr,"cross-linked at cluster %d\n",clusterstart);
	    return *chain;		// cross-linked! go home
	}
	clusterstart = fat0.next_cluster(clusterstart);
    } while(clusterstart!=fat_FAT::fatEOF);
    return *chain;
}

/* fwrite_dir:
 * Get the directory and print it.
 * If sector, just get for that sector.
 * If cluster, do the whole clusterchain.
 */

void	ffs::fwrite_dir(FILE *f,pos loc, int entry,
			fat_dirent *odir,
			dir_callback *dcb,
			int flags)
{
    /*
     * loc is a sector.
     * This is the simple case. 
     *  Just get the 16 entries for the sector and show them
     */
     

    if(loc.is_sector()){
	uchar buf[512];

	if(img->readblock(loc.sector(&bpb),buf)==0) return;

	fat_dirent *dir[16];
	int valid=0;
	int j;
	for(j=0;j<16;j++){
	    dir[j] = new fat_dirent(&bpb);
	    if(dir[j]->readbuf(buf+32*j)){
		valid++;
	    }
	}
	    
	for(j=0;j<16;j++){
	    if(dir[j]->valcode==fat_dirent::VALID || (flags & DIR_SHOWLDIR)){
		if(f) dir[j]->fwrite(f);
		if(dcb){
		    if(dir[j]->is_dir()) dcb->dir(dir[j]);	// note that it was found
		    else                 dcb->file(dir[j]);
		}
	    }

	    if(j==entry && odir!=0) odir = dir[j];
	}

	for(j=0;j<16;j++){
	    delete dir[j];
	}
	return;
    }

    /* loc is a cluster.
     * this is a harder case, because the clusterchain might be bad.
     * At each point along the chain we need to validate that the buffer is
     * really a directory.
     *
     * TK: Currently, grab up to 1MB of directory. However, what we really should
     * do is modify write_clusterchain() so that it doesn't repeat the same block twice.
     */
    uchar *buf=0;
    int   buflen=0;
    uchar  *dp;
    buflen = write_clusterchain(0,&buf,loc.cluster(),32 * 65536);

    if(buf){
	fat_dirent dir(&bpb);		// directory entry read
					// needed to keep state across blocks
					// to properly handle long directories

	dir.init();			// zero 
	int j=0;
	for(int pos=0;pos<buflen;pos+=512){	// pos will skip for each block

	    /* Verify that this block is good.
	     * Note -- verification is done with dir.readbuf() as well, so we
	     * are actually calling it twice
	     */
	    if(analyze_isdirblock(buf+pos)==false){ // ran out of directory
		if(opt_v){
		    printf("analyize_isdirblock() fails at pos=%d\n",pos);
		}
		break;
	    }
	    for(dp=buf+pos;dp<buf+buflen && dp<buf+pos+512;dp+=32){	// dir pointer
		dir.readbuf(dp);	// get next part
		if(dir.is_complete()){
		    if(f) dir.fwrite(f);
		    if(dcb){
			if(dir.is_dir()) dcb->dir(&dir);
			else             dcb->file(&dir);
		    }
		    dir.init();
		}
	    }
	}
	if(dir.is_null()==false){	// write out the incomplete entry
	    if(f) dir.fwrite(f);
	    if(dir.is_dir() && dcb) dcb->dir(&dir);
	}
	free(buf);
    }
}


/****************************************************************
 ** The analysis routines
 ****************************************************************/


/* analyze_isdirblock:
 * Make an educated guess on whether buf corresponds to a directory or not
 */


bool ffs::analyze_isdirblock(const uchar buf[512])
{
    fat_dirent d0(&bpb);		// create a dirent record for position 0
    bool ret = d0.readbuf(buf);		// attempt to read it out of buf
    if(ret==false) return false;	// couldn't read a directory at position 0

    /* Try to read the 15 other directory entries that might be in this block.
     * If they all have the same name, size and ctime, then this is probably
     * not a valid directory block.
     */
    fat_dirent d(&bpb);
    int i;
    bool all_same = true;
    for(i=1;i<16 && all_same;i++){
	ret = d.readbuf(buf+32*i);	// read this value
	if(ret){
	    if(d0.similar(d)==false){
		all_same = false;
	    }
	}
    }
    if(all_same) return false;		// not likely a directory
    return true;
}

// does cluster i look like a directory?
bool   ffs::analyze_isdircluster(cluster_t i) 
{
    uchar buf[512];
    sector_t s = bpb.cluster2sector(i);	// get the sector number
    if(img->readblock(s,buf)!=sizeof(buf)) return false; // couldn't read
    return analyze_isdirblock(buf);
}
