/* The forensic file system master include file.
 *
 */

#ifndef FFS_H
#define FFS_H

#include "ftypes.h"

#include "ffs_chain.h"
#include "pos.h"
#include "img.h"
#include "blockmap.h"
#include "extension_db.h"
#include "fat_FAT.h"
#include "fat_BPB.h"
#include "fat_dirent.h"
#include "sorted.h"



class intel {
public:;
    /* functions to decode intel 2, 3 and 4 unsigned byte values */
    static inline int u1(const uchar *p)
    {
	return p[0];
    }

    static inline int u2(const uchar *p)
    {
	return p[0] | p[1]<<8;
    }

    static inline int u4(const uchar *p)
    {
	return p[0] | p[1]<<8 | p[2]<<16 | p[3]<<24;
    }
};

class ffs {
public:;
    /* OPTIONS MANAGEMENT */
    static int debug;			// are we debugging?
    static int opt_v;			// verbose option
    static int opt_q;			// quote option
    static void  set_debug(int val) { debug=val;}
    static void  set_verbose(int val) { opt_v=val;}
    static void  set_quiet(int val) { opt_q=val;}

    /* USEFUL FUNCTIONS */
    static char *itocomma(char *buf,unsigned i); // display a number with commas
    static void  dump_tm(FILE *out,struct tm *tm); // dump a tm to out in readable form

    /* FINALLY - THE FORENSIC STUFF */
    class img     *img;			// where we are getting our disk to analyze
    ffs(class img *img_);		// create an FFS

    /* Routines for finding and manipulating BPBs. */
    class  fat_BPB bpb;			// the BPB from which this ffs was read
    sector_t  BPB_loc;			// where we found it
    bool   find_BPB(sector_t start);	// searches for the BPB, starting a block

    /* Routines for manipulating FAT and FAT clusterchains.
     * (The cluster# is the block number where the file start.
     * The FAT is used to determine the next cluster number.)
     * As a side effect, this means that we can recover data even when the FAT is
     * lost.
     */
    class  fat_FAT fat0;			// our fat
    int    get_cluster(cluster_t cluster,uchar *buf,int buflen);	// get contents of a single cluster;
					// returns # of bytes written

    int    write_clusterchain(FILE *out,uchar **ptr,
			      cluster_t start,ulong maxbytes); // returns bytes written
    void   fwrite_cluster(FILE *f,cluster_t cluster);          // does the entire cluster
    void   fprint_cluster(FILE *f,cluster_t cluster);
    class  img::sectorchain &get_sectorchain(cluster_t clusterstart); // gets a sectorchain for a cluster
    class  ffs_chain &get_clusterchain(cluster_t clusterstart); // gets a clusterchain for a cluster

    /*
     * Routines for manipulating directories
     */

    class dir_callback {
    public:;
	dir_callback() {};
	virtual void dir(class fat_dirent *dirent) {}; // called for each dir
	virtual void file(class fat_dirent *dirent){}; // called for each file
    };

    void   fwrite_dir(FILE *f,pos loc,int entry,fat_dirent *dir,dir_callback *dcb,int flags);
    enum {
	DIR_SHOWLDIR=0x0001
    };

    /* Analysis routines */
    static bool analyze_islevel1(const char *filename); // true if filename looks like a level1 name
    bool   analyze_isdirblock(const uchar buf[512]);// Returns true of buf looks like a directory block
    bool   analyze_isdircluster(cluster_t i); // does cluster i look like a directory?
    void   analyze_block(int block);	// analyizes a block
    void   analyze_dir(int block);

};

#endif
