/*
 * img.cpp:
 * The disk image system.
 */

#include "ffs.h"
#include "fatdump.h"

#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>

#ifndef O_BINARY
#define O_BINARY 0
#endif


img::img()
{
    first_nonblank = -1;
}



void img::block2pp(const unsigned char block[512],char buf[550])
{
    int i;
    for(i=0;i<512;i++){
	unsigned char ch = *block++;
	if(ch<32 || ch>126) *buf++ ='.';
	else *buf++=ch;

	if((i+1)%64==0) *buf++ = '\n';
    }
    *buf++ = 0;
}


/* Disk manipulation routines */

bool img::openimage(const char *diskfile)
{
    image_size = 0;
    fd = open(diskfile,O_RDONLY|O_BINARY);
    if(fd<0){
	return false;
    }

    image_size   = lseek(fd,0,SEEK_END);
    lseek(fd,0,SEEK_SET);

	
    if(image_size==0){
	fprintf(stderr,"fatdump: %s has no data!\n",diskfile);
	return false;
    }

    image_blocks = image_size / 512;

    return true;
}

bool img::blank_block(const uchar *buf)
{
    register int i;
    for(i=0;i<512;i++){
	if(buf[i]!=0) return false;	// block is not blank
    }
    return true;			// block is blank
}

int img::readblock(sector_t blocknum,uchar *buf)
{
    return readblocks(blocknum,buf,1);	// just read one block
}

int img::readblocks(sector_t blocknum,uchar *buf,int blockcount)
{
    long pos   = blocknum  * 512;
    int  bytes = 512*blockcount;

    lseek(fd, pos, SEEK_SET);
    int r = read(fd, buf, bytes);

    /* Check each complete block read and see if it is a blank block.
     * If so, remember this fact.
     */
    for(int i=0;i<blockcount;i++){
	sector_t blki = blocknum+i;
	if(first_nonblank !=-1 && blki >= first_nonblank){ // no need to look anymore
	    break;
	}
	const uchar *bufi    = buf+i*512;  

	if(blank_block(bufi)==false){
	    first_nonblank = blki;	// I found an earlier non-blank block!
	    break;
	}
    }

    if(opt_v > 2 && !opt_i){
	for(int i=0;i<blockcount && i*512<r;i++){
	    sector_t blki = blocknum+i;
	    printf("======= block %d =======\n",blki);
	    char ppbuf[600];
	    block2pp(buf+i*512,ppbuf);
	    puts(ppbuf);
	}
    }
    return r;
}

#ifndef MIN
#define MIN(x,y) ((x)<(y) ? (x) : (y))
#endif

int img::write(int fd,class img::sectorchain *chain)
{
    int written=0;
    int i;

    for(i=0;i<chain->count();i++){
	unsigned char buf[512];
	if(readblock(chain->count(),buf)==0){
	    return -1;			// bad
	}
	int to_write = MIN(512,chain->bytes() - written);
	int r = ::write(fd,buf,to_write);
	if(r<0){
	    return -1;			// bad again
	}
	written += r;
	if(r!=to_write){
	    break;			// bad
	}
    }
    return written;
}

