/*
 * img.h:
 * The disk image driver.
 * This class:
 * -- allows us access to a disk file as if it were a block device.
 * -- allows us to refer to a chain of blocks (with a length)
 * -- allows us to write a chain of blocks to an output file
 */

#ifndef IMG_H
#define IMG_H



class img {
public:

    // block2pp is a pretty-print routine.
    // it takes a 512-byte block and puts its results in a buffer
    // somewhat larger...
    static void block2pp(const unsigned char block_buf[512],char buf[550]);
    static bool blank_block(const unsigned char *buf); // return true if block is blank

    class sectorchain : public ffs_chain {
	unsigned int bytes_;			// number of bytes in the chain
    public:
	sectorchain() { bytes_=0;}
	unsigned int bytes() { return bytes_;}
    };


    // These are the class routines
    //
    img();
    int    fd;				// image file
    bool   openimage(const char *fn);	// open a disk file
    int    readblock(unsigned long blocknum,uchar *block_buf); // returns bytes read
    int    readblocks(unsigned long blocknum,uchar *block_buf,int count); // returns bytes read
    unsigned long image_size;		// how big the fat image is
    unsigned long image_blocks;		// how big the fat image is
    int      first_nonblank;		// location of first non-blank sector (not offset)
    int      write(int fd,class sectorchain *chain);	// returns # of bytes written
};

#endif

